# -*- coding: utf-8 -*-

from collective.taxonomy.interfaces import ITaxonomy
from imio.directory.core.contents import IContact
from imio.directory.core.utils import get_entity_uid_for_contact
from imio.smartweb.common.utils import translate_vocabulary_term
from plone.app.contenttypes.indexers import _unicode_save_string_concat
from plone.indexer import indexer
from Products.CMFPlone.utils import safe_unicode
from zope.component import getSiteManager


@indexer(IContact)
def translated_in_nl(obj):
    return bool(obj.title_nl)


@indexer(IContact)
def translated_in_de(obj):
    return bool(obj.title_de)


@indexer(IContact)
def translated_in_en(obj):
    return bool(obj.title_en)


@indexer(IContact)
def title_nl(obj):
    if not obj.title_nl:
        raise AttributeError
    return obj.title_nl


@indexer(IContact)
def title_de(obj):
    if not obj.title_de:
        raise AttributeError
    return obj.title_de


@indexer(IContact)
def title_en(obj):
    if not obj.title_en:
        raise AttributeError
    return obj.title_en


@indexer(IContact)
def description_nl(obj):
    if not obj.description_nl:
        raise AttributeError
    return obj.description_nl


@indexer(IContact)
def description_de(obj):
    if not obj.description_de:
        raise AttributeError
    return obj.description_de


@indexer(IContact)
def description_en(obj):
    if not obj.description_en:
        raise AttributeError
    return obj.description_en


@indexer(IContact)
def container_uid(obj):
    uid = get_entity_uid_for_contact(obj)
    return uid


def get_searchable_text(obj, lang):
    topics = []
    for topic in getattr(obj.aq_base, "topics", []) or []:
        topics.append(
            translate_vocabulary_term("imio.smartweb.vocabulary.Topics", topic)
        )

    sm = getSiteManager()
    utility = sm.queryUtility(ITaxonomy, name="collective.taxonomy.contact_category")
    categories = []
    for category in getattr(obj.aq_base, "taxonomy_contact_category", []) or []:
        categories.append(
            utility.translate(category, context=obj, target_language=lang)
        )
    subjects = obj.Subject()
    title_field_name = "title"
    description_field_name = "description"
    subtitle_field_name = "subtitle"
    if lang != "fr":
        title_field_name = f"{title_field_name}_{lang}"
        description_field_name = f"{description_field_name}_{lang}"
        subtitle_field_name = f"{subtitle_field_name}_{lang}"

    result = " ".join(
        (
            safe_unicode(getattr(obj, title_field_name)) or "",
            safe_unicode(getattr(obj, subtitle_field_name)) or "",
            safe_unicode(getattr(obj, description_field_name)) or "",
            *topics,
            *categories,
            *subjects,
        )
    )
    return _unicode_save_string_concat(result)


@indexer(IContact)
def SearchableText_fr_contact(obj):
    return get_searchable_text(obj, "fr")


@indexer(IContact)
def SearchableText_nl_contact(obj):
    return get_searchable_text(obj, "nl")


@indexer(IContact)
def SearchableText_de_contact(obj):
    return get_searchable_text(obj, "de")


@indexer(IContact)
def SearchableText_en_contact(obj):
    return get_searchable_text(obj, "en")



# -*- coding: utf-8 -*-

from imio.directory.core.contents import IContact
from imio.directory.core.interfaces import IImioDirectoryCoreLayer
from imio.smartweb.common.rest.utils import get_restapi_query_lang
from plone.app.contentlisting.interfaces import IContentListingObject
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.serializer.dxcontent import SerializeFolderToJson
from plone.restapi.serializer.summary import DefaultJSONSummarySerializer
from Products.CMFCore.WorkflowCore import WorkflowException
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(ISerializeToJson)
@adapter(IContact, IImioDirectoryCoreLayer)
class SerializeContactToJson(SerializeFolderToJson):
    def __call__(self, version=None, include_items=True):
        result = super(SerializeContactToJson, self).__call__(
            version, include_items=True
        )
        version = "current" if version is None else version
        obj = self.getVersion(version)

        query = self.request.form
        lang = get_restapi_query_lang(query)
        if lang and lang != "fr":
            result["title"] = getattr(obj, f"title_{lang}")
            result["subtitle"] = getattr(obj, f"subtitle_{lang}")
            result["description"] = getattr(obj, f"description_{lang}")

        # maybe not necessary :
        result["title_fr"] = obj.title
        result["subtitle_fr"] = obj.subtitle
        result["description_fr"] = obj.description

        result["is_geolocated"] = obj.is_geolocated
        return result


@implementer(ISerializeToJsonSummary)
@adapter(Interface, IImioDirectoryCoreLayer)
class ContactJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self):
        summary = super(ContactJSONSummarySerializer, self).__call__()

        query = self.request.form
        lang = get_restapi_query_lang(query)
        if lang == "fr":
            # nothing to go, fr is the default language
            return summary

        obj = IContentListingObject(self.context)
        for orig_field in ["title", "description"]:
            field = f"{orig_field}_{lang}"
            accessor = self.field_accessors.get(field, field)
            value = getattr(obj, accessor, None)
            try:
                if callable(value):
                    value = value()
            except WorkflowException:
                summary[orig_field] = None
                continue
            if orig_field == "description" and value is not None:
                value = value.replace("**", "")
            summary[orig_field] = json_compatible(value)

        return summary



# -*- coding: utf-8 -*-

from imio.directory.core.testing import IMIO_DIRECTORY_CORE_FUNCTIONAL_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.zope import Browser

import transaction
import unittest


class TestLocalRoles(unittest.TestCase):
    layer = IMIO_DIRECTORY_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.entity = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            title="Entity",
        )
        self.contact = api.content.create(
            container=self.entity,
            type="imio.directory.Contact",
            title="Contact",
        )

    def test_local_manager_in_sharing(self):
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/@@sharing".format(self.entity.absolute_url()))
        content = browser.contents
        self.assertIn("Can manage locally", content)

        browser.open("{}/@@sharing".format(self.contact.absolute_url()))
        content = browser.contents
        self.assertNotIn("Can manage locally", content)



# -*- coding: utf-8 -*-

from eea.facetednavigation.subtypes.interfaces import IFacetedNavigable
from imio.directory.core.contents import IEntity
from imio.directory.core.testing import IMIO_DIRECTORY_CORE_INTEGRATION_TESTING
from plone import api
from plone.api.exc import InvalidParameterError
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import queryUtility

import unittest


class TestEntity(unittest.TestCase):
    layer = IMIO_DIRECTORY_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.authorized_types_in_entity = ["imio.directory.Contact"]
        self.unauthorized_types_in_entity = [
            "imio.directory.Entity",
            "Folder",
            "Page",
            "Link",
        ]

        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_ct_entity_schema(self):
        fti = queryUtility(IDexterityFTI, name="imio.directory.Entity")
        schema = fti.lookupSchema()
        self.assertEqual(IEntity, schema)

    def test_ct_entity_fti(self):
        fti = queryUtility(IDexterityFTI, name="imio.directory.Entity")
        self.assertTrue(fti)

    def test_ct_entity_factory(self):
        fti = queryUtility(IDexterityFTI, name="imio.directory.Entity")
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            IEntity.providedBy(obj),
            "IEntity not provided by {0}!".format(
                obj,
            ),
        )

    def test_ct_entity_adding(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        entity = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            title="entity",
        )
        self.assertTrue(
            IEntity.providedBy(entity),
            "IEntity not provided by {0}!".format(
                entity.id,
            ),
        )

        self.assertTrue(IFacetedNavigable.providedBy(entity))

        parent = entity.__parent__
        self.assertIn("entity", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=entity)
        self.assertNotIn("entity", parent.objectIds())

    def test_ct_entity_globally_addable(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="imio.directory.Entity")
        self.assertTrue(fti.global_allow, "{0} is not globally addable!".format(fti.id))

    def test_ct_entity_filter_content_type(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        entity = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            title="entity",
        )
        for t in self.unauthorized_types_in_entity:
            with self.assertRaises(InvalidParameterError):
                api.content.create(
                    container=entity,
                    type=t,
                    title="My {}".format(t),
                )
        for t in self.authorized_types_in_entity:
            api.content.create(
                container=entity,
                type=t,
                title="My {}".format(t),
            )



# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import ILocalManagerAware
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import schema
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform import directives
from plone.dexterity.content import Container
from plone.supermodel import model
from zope.interface import implementer


class IEntity(model.Schema):
    """Marker interface and Dexterity Python Schema for Entity"""

    directives.widget(zip_codes=SelectFieldWidget)
    zip_codes = schema.List(
        title=_("Zip codes and cities"),
        description=_("Choose zip codes for this entity"),
        value_type=schema.Choice(vocabulary="imio.smartweb.vocabulary.Cities"),
    )


@implementer(IEntity, ILocalManagerAware)
class Entity(Container):
    """Entity content type"""



# -*- coding: utf-8 -*-

from imio.smartweb.common.browser.forms import CustomAddForm
from imio.smartweb.common.browser.forms import CustomEditForm
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.dexterity.browser.add import DefaultAddView
from plone.z3cform import layout
from zope.i18n import translate


class ContactCustomAddForm(CustomAddForm):
    portal_type = "imio.directory.Contact"

    def updateWidgets(self):
        super(ContactCustomAddForm, self).updateWidgets()
        if "IGeolocatable.geolocation" in self.widgets:
            self.widgets["IGeolocatable.geolocation"].description = _(
                "The geolocation is generated on the basis of the address during contact save. "
                "It is possible to change the pointer manually if it is not correctly positioned."
            )


class ContactCustomAddView(DefaultAddView):
    form = ContactCustomAddForm


class ContactCustomEditForm(CustomEditForm):
    def updateWidgets(self):
        super(ContactCustomEditForm, self).updateWidgets()
        if "IGeolocatable.geolocation" in self.widgets:
            self.widgets["IGeolocatable.geolocation"].description = _(
                "The geolocation is generated on the basis of the address during contact save. "
                "It is possible to change the pointer manually if it is not correctly positioned."
            )
        if "ILeadImageBehavior.image" in self.widgets:
            language = api.portal.get_current_language()
            desc = self.widgets["ILeadImageBehavior.image"].description
            desc = translate(desc, target_language=language)
            prepend_desc = _("Example : Photo of a person or organization building")
            prepend_desc = translate(prepend_desc, target_language=language)
            self.widgets[
                "ILeadImageBehavior.image"
            ].description = f"{prepend_desc}. {desc}"


ContactCustomEditView = layout.wrap_form(ContactCustomEditForm)



# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import IImioSmartwebCommonLayer
from plone.theme.interfaces import IDefaultPloneLayer


class IImioDirectoryCoreLayer(IImioSmartwebCommonLayer, IDefaultPloneLayer):
    """Marker interface that defines a browser layer."""



# -*- coding: utf-8 -*-

from imio.directory.core.contents.entity.content import IEntity
from imio.smartweb.common.utils import translate_vocabulary_term
from Products.CMFPlone.utils import parent

import base64
import vobject


def get_entity_uid_for_contact(contact):
    obj = contact
    while not IEntity.providedBy(obj):
        obj = parent(obj)
    entity = obj
    return entity.UID()


def get_vcard(contact):
    vcard = vobject.vCard()

    vcard.add("fn")
    vcard.fn.value = contact.Title()
    emails = contact.mails or []
    for email in emails:
        vcard.add("email")
        vcard.email.type_param = email.get("type")
        vcard.email.value = email.get("mail_address")

    phones = contact.phones or []
    for phone in phones:
        vcard.add("tel")
        vcard.tel.type_param = phone.get("type")
        vcard.tel.value = phone.get("number")

    urls = contact.urls or []
    for url in urls:
        vcard.add("url")
        vcard.url.type_param = url.get("type")
        vcard.url.value = url.get("url")

    addr = {
        "number": contact.number or "",
        "street": contact.street or "",
        "additional": contact.complement or "",
        "zipcode": str(contact.zipcode or ""),
        "city": contact.city or "",
    }
    if any(addr.values()):
        # If any field values (except country) has been filled in, we consider
        # that an address has been encooded
        vcard.add("adr")
        vcard.adr.value = vobject.vcard.Address(
            street=addr.get("street"),
            city=addr.get("city"),
            region="",
            code=addr.get("zipcode"),
            country=translate_vocabulary_term(
                "imio.smartweb.vocabulary.Countries", contact.country
            ),
            box=addr.get("number"),
            extended=addr.get("additional"),
        )
    if contact.logo:
        o = vcard.add("PHOTO;ENCODING=b;TYPE=image/jpeg")
        base64_bytes = base64.b64encode(contact.logo.data)
        o.value = base64_bytes.decode("utf-8")
    return vcard
